/*
 * configs.c  - dlevette
 *
 * Routines for handling config files
 *
 *
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "raw_stuff.h"

extern int IP_MODE;
extern int TCP_MODE;
extern int UDP_MODE;
extern int ICMP_MODE;
extern int ARP_MODE;
extern int WRITE_CONFIG;

extern u_short src_port;
extern u_short dest_port;
extern u_short window, u_priority, ttl, ip_id, frag_off;
extern char *src_addr, *dest_addr;
extern char *src_mac, *dest_mac;

extern u_short arp_optc;
extern char *send_ip, *targ_ip;
extern char *send_mac, *targ_mac;

extern u_short icmp_type;
extern u_short icmp_code;

extern struct config_entrys *build_conf;

struct config_entrys default_ip [] =  
{
	/* Etherlink */
	{"SRC_MAC", "00:00:00:00:00:00"},   
	{"DEST_MAC", "FF:FF:FF:FF:FF:FF"},  
	
	/* IP */
	{"SRC_IP", "127.0.0.1"},  
	{"DEST_IP", "127.0.0.1"},   
	{"IP_ID", "00001"},     
	{"FRAG_OPTS", "d"},
	{"TTL", "250"},
	{NULL, NULL}
};

struct config_entrys default_tcp [] =  
{
	/* Etherlink */
	{"SRC_MAC", "00:00:00:00:00:00"},   
	{"DEST_MAC", "FF:FF:FF:FF:FF:FF"},  
	
	/* IP */
	{"SRC_IP", "127.0.0.1"},  
	{"DEST_IP", "127.0.0.1"},   
	{"IP_ID", "00001"},     
	{"FRAG_OPTS", "d"},
	{"TTL", "250"},
	
	/* TCP */
	{"SRC_PORT", "55555"},  
	{"DEST_PORT", "66666"}, 	
	{"TCP_FLAGS", "SAFU"},	
	{"WINDOW_SIZE", "00535"},  
	{"URG_POINTER", "0"},	
	{NULL, NULL}		
};

struct config_entrys default_udp [] =  
{
	/* Etherlink */
	{"SRC_MAC", "00:00:00:00:00:00"},   
	{"DEST_MAC", "FF:FF:FF:FF:FF:FF"},  
	
	/* IP */
	{"SRC_IP", "127.0.0.1"},    
	{"DEST_IP", "127.0.0.1"},   
	{"IP_ID", "0"},     
	{"FRAG_OPTS", "d"},
	{"TTL", "50"},  
	
	/* UDP */
	{"SRC_PORT", "5555"},
	{"DEST_PORT", "6666"},
	{NULL, NULL}
};

struct config_entrys default_icmp [] =  
{
	/* Etherlink */
	{"SRC_MAC", "00:00:00:00:00:00"},   
	{"DEST_MAC", "FF:FF:FF:FF:FF:FF"},  
	
	/* IP */
	{"SRC_IP", "127.0.0.1"},    
	{"DEST_IP", "127.0.0.1"},   
	{"IP_ID", "0"},     
	{"FRAG_OPTS", "d"},
	{"TTL", "50"},  
	
	/* ICMP */
	{"ICMP_TYPE", "0"},
	{"ICMP_CODE", "0"},
	{NULL, NULL}	
};

struct config_entrys default_arp [] =  
{
	/* Etherlink */
	{"SRC_MAC", "00:00:00:00:00:00"},   
	{"DEST_MAC", "FF:FF:FF:FF:FF:FF"},  
	
	/* ARP/RARP */
	{"ARP_TYPE", "1"},     /* ARP request */
	{"SENDER_MAC", "AA:BB:22:BB:44:FF"},   
	{"SENDER_IP", "127.0.0.1"},
	{"TARGET_MAC", "00:00:00:00:00:00"},   
	{"TARGET_IP", "127.0.0.1"},
	{NULL, NULL}	
};

/*
 * Returns appropiate default packet
 * FIXME: Assign sane defaults
*/
struct config_entrys *get_default_config()
{
	struct config_entrys *ret;

	if(IP_MODE == 1) ret = default_ip;
	if(TCP_MODE == 1) ret = default_tcp;
	if(UDP_MODE == 1) ret = default_udp;
	if(ICMP_MODE == 1) ret = default_icmp;
	if(ARP_MODE == 1) ret = default_arp;
	
	return(ret);	
	
}

void get_config(char *file)
{
	FILE *r_fd;
	char line[MAXLINE];
	char *option = NULL;        /* Config option */
	char *parameter = NULL;     /* User defined value for option */
	
	int i;
	int sane;
	
	if((r_fd = fopen(file, "r")) == NULL)
		error_msg("Unable to open config file\n");

	while(fgets(line, MAXLINE, r_fd))
	{	/* Skip empty lines and comments */
		if(line[0] == '#' || line[0] == '\n') continue;
			
		i = 0;
		while(build_conf[i].option != NULL)
		{	/* Check given option is valid */
			if((option = strstr(line, build_conf[i].option)) != NULL)
			{	/* Grab option value */
				parameter = strtok(line, " \t=\t ");
				parameter =  strtok(NULL, " = ");
				
				/* Perform validation on values */
				if((sane = chk_config(option, parameter)) == -1)
					error_msg("Incorrect option value supplied\n");
			}
			
		i++;
		}
	}
	
	fclose(r_fd);
}

int chk_config(char *option, char *value)
{
	chomp(value);
	if((strcmp("SRC_MAC", option)) == 0 )
	{
		if(chk_mac(value, ETHER_LEN) != -1){
			src_mac = malloc(strlen(value));
			memcpy(src_mac, value, strlen(value)); 
		}
		else return -1;
	}
	else if((strcmp("DEST_MAC", option)) == 0)
	{
		if(chk_mac(value, ETHER_LEN) != -1){
			dest_mac = malloc(strlen(value));
			memcpy(dest_mac, value, strlen(value));
		}
		else return -1;
	}
	else if((strcmp("SRC_IP", option)) == 0 )
	{
		if(chk_ip(value) != -1){
			src_addr = malloc(strlen(value));
			memcpy(src_addr, value, strlen(value));
		}
		else return -1;
	}
	else if((strcmp("DEST_IP", option)) == 0)
	{
		if(chk_ip(value) != -1){
			dest_addr = malloc(strlen(value));
			memcpy(dest_addr, value, strlen(value));
		}
		else return -1;
	}
	else if((strcmp("SRC_PORT", option)) == 0 )
	{
		if(chk_int(0, 65535, atoi(value)) != -1)
			src_port = atoi(value);
		else
			return -1;
	}
	else if((strcmp("DEST_PORT", option)) == 0)
	{
		if(chk_int(0, 65535, atoi(value)) != -1)
			dest_port = atoi(value);
		else
			return -1;
	}
	else if((strcmp("WINDOW", option)) == 0 )
	{
		if(chk_int(0, 65535, atoi(value)) != -1)
			window = atoi(value);
		else
			return -1;
	}
	else if((strcmp("IP_ID", option)) == 0)
	{
		if(chk_int(0, 65535, atoi(value)) != -1)
			ip_id = atoi(value);
		else
			return -1;
	}
	else if((strcmp("TTL", option)) == 0)
	{
		if(chk_int(0, 255, atoi(value)) != -1)
			ttl = atoi(value);
		else
			return -1;
	}
	else if((strcmp("URG_POINTER", option)) == 0)
	{
		if(chk_int(0, 3, atoi(value)) != -1)
			u_priority = atoi(value);
		else
			return -1;		
	}
	else if((strcmp("TCP_FLAGS", option)) == 0)
	{
		if(assign_tcp_flags(value) == -1)
			return -1;
	}
	else if((strcmp("FRAG_OPTS", option)) == 0)
	{
		if(assign_frag_opts(value) == -1)
			return -1;
	}
	if((strcmp("SENDER_MAC", option)) == 0 )
	{
		if(chk_mac(value, ETHER_LEN) != -1){
			send_mac = malloc(strlen(value));
			memcpy(send_mac, value, strlen(value)); 
		}
		else return -1;
	}
	else if((strcmp("TARGET_MAC", option)) == 0)
	{
		if(chk_mac(value, ETHER_LEN) != -1){
			targ_mac = malloc(strlen(value));
			memcpy(targ_mac, value, strlen(value));
		}
		else return -1;
	}
	else if((strcmp("SENDER_IP", option)) == 0 )
	{
		if(chk_ip(value) != -1){
			send_ip = malloc(strlen(value));
			memcpy(send_ip, value, strlen(value));
		}
		else return -1;
	}
	else if((strcmp("TARGET_IP", option)) == 0)
	{
		if(chk_ip(value) != -1){
			targ_ip = malloc(strlen(value));
			memcpy(targ_ip, value, strlen(value));
		}
		else return -1;
	}
	else if((strcmp("ARP_TYPE", option)) == 0)
	{
		if(chk_int(1, 4, atoi(value)) != -1 ||
		   chk_int(8, 10, atoi(value)) != -1)
			   arp_optc = atoi(value);
		else return -1;
	}
	else if((strcmp("ICMP_TYPE", option)) == 0)
	{
		if(chk_int(0, 0, atoi(value)) != -1 ||
		   chk_int(3, 5, atoi(value)) != -1 ||
		   chk_int(8, 8, atoi(value)) != -1 ||
		   chk_int(11, 18, atoi(value)) != -1)
			   icmp_type = atoi(value);
		else return -1;
	}
	else if((strcmp("ICMP_CODE", option)) == 0)
	{
		if(chk_int(1, 15, atoi(value)) != -1)
			   icmp_code = atoi(value);
		else return -1;
	}
	
	if((create_config(option, value)) == -1)
		error_msg("Unable to create configuration structure\n");
	
	return 0;
}

int create_config(char *option, char *value)
{
	int i = 0;
	while(build_conf[i].option != NULL)
	{
		if((strcmp(build_conf[i].option, option)) == 0)
		{
			build_conf[i].value = malloc(strlen(value));
			memcpy(build_conf[i].value, value, strlen(value));
			printf("%s::%s\n",option, build_conf[i].value);
		}
	i++;
	}
	return 0;
}


int write_config(char *write_file)
{
	FILE *w_fd;
	
	if((w_fd = fopen(write_file, "a")) == NULL)
		error_msg("Unable to open/create file for writing\n");
	
	int i = 0;
	while(build_conf[i].option != NULL)
	{
		fprintf(w_fd, "%s =  %s\n", build_conf[i].option, build_conf[i].value);
		i++;
	}

	fclose(w_fd);
	return 0;
}
