/*
 * main.c  -dlevette
 *
 *
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <unistd.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <net/ethernet.h>
#include <netinet/tcp.h>
#include <netinet/udp.h>
#include <linux/icmp.h>
#include <netinet/in.h>
#include <net/if_arp.h>

#include "raw_stuff.h"

/* Mode Flags */
int IP_MODE = 0;
int TCP_MODE  = 0;
int UDP_MODE  = 0;
int ICMP_MODE = 0;
int ARP_MODE  = 0;
int FUZZ_MODE = 0;
int GOT_IFACE = 0;
int HAVE_CONFIG = 0;
int WRITE_CONFIG = 0;

/* TCP Flags */
int TCP_FIN = 0;
int TCP_SYN = 0;
int TCP_RST = 0;
int TCP_PSH = 0;
int TCP_ACK = 0;
int TCP_URG = 0;

u_short src_port = 0;  /* FIXME: randomize between 1025 - 65535 */
u_short dest_port = 0, i = 0;
u_short window = 512, u_priority, ttl, ip_id = 0;
char *src_addr, *dest_addr = NULL;
char *src_mac, *dest_mac = NULL;
int frag_off = 0;

u_short arp_optc;
char *send_ip, *targ_ip = NULL;
char *send_mac, *targ_mac = NULL;

u_short icmp_type = 0;
u_short icmp_code = 0;

struct config_entrys *build_conf;
char *write_file;

void usage()
{
	printf("Usage: pkmap -i {iface} {mode} [mode opts] {ip options} {ether options}\n\n");
	printf("Try: pkmap -h for help\n\n");
	exit(-1);
}

void error_msg(char *msg)
{
	fprintf(stderr, "Error: %s", msg);
	exit(-1);
}

int assign_tcp_flags(char *flags)
{
	int i = 0;
	while(flags[i] != '\0')
	{
		if(flags[i] == 'F') TCP_FIN = 1;
		else if(flags[i] == 'S') TCP_SYN = 1;
		else if(flags[i] == 'R') TCP_RST = 1;
		else if(flags[i] == 'P') TCP_PSH = 1; 
		else if(flags[i] == 'A') TCP_ACK = 1;
		else if(flags[i] == 'U') TCP_URG = 1;
		else return -1;
	i++;
	}
	return 0;	
}

int assign_frag_opts(char *opts)
{
	int i = 0;
	while(opts[i] != '\0')
	{
		if(opts[i] == 'r')
			frag_off += IP_RF;
		else if(opts[i] == 'd')
			frag_off += IP_DF;
		else if(opts[i] == 'm')
			frag_off += IP_MF;
		else return -1;
	i++;
	}
	return 0;
}

int main(int argc, char **argv)
{
	char *iface = "lo";
        
	u_short icmp_id = 0;
	u_short icmp_seq = 0;
	
	/* Header structures */
        struct ether_header *ether_hdr;
        struct ip *ip_hdr = NULL;
        struct tcphdr *tcp_hdr;
	struct udphdr *udp_hdr;
	struct icmphdr *icmp_hdr;
	struct arphdr *arp_hdr;
	struct arp_ethip_hdr *arp_ip_hdr;
	
	/* Packet stuff */
	u_char *packet = NULL;
	int packetlen = 0;
	int num_pk = 1;
	
	/* Payload stuff */
	FILE *r_fd;
	char *payload = NULL;
	int pay_size = 0;
	
	/* Config stuff */
	char *config_file = NULL;
	char *write_file = NULL;
	
	int rsock = 0;
	
    
	int opt;
	int i = 0;
	while((opt = getopt(argc, argv, 
		"hVPTUMAC:W:i:n:s:d:f:w:u:S:D:I:F:L:t:m:o:g:j:G:J:Z:Y:p:")) != -1){
		switch(opt){
			case 'h':
				if(IP_MODE == 1) ip_help();
				else if(TCP_MODE == 1) tcp_help();
				else if(UDP_MODE == 1) udp_help();
				else if(ICMP_MODE == 1) icmp_help();
				else if(ARP_MODE == 1) arp_help();
				else help();
				exit(0);
			case 'V':
				printf("%s-%s\n", APP, VERSION);
				exit(0);
			case 'P':
				IP_MODE = 1;
				build_conf = get_default_config();
				break;
			case 'T':
				TCP_MODE = 1;
				build_conf = get_default_config();
				break;
			case 'U':
				UDP_MODE = 1;
				build_conf = get_default_config();
				break;
			case 'M':
				ICMP_MODE = 1;
				build_conf= get_default_config();
				break;
			case 'A':
				ARP_MODE = 1;
				build_conf = get_default_config();
				break;
			case 'C':
				HAVE_CONFIG = 1;
				config_file = optarg;
				break;
			case 'W':
				WRITE_CONFIG = 1;
				write_file = optarg;
				break;
			case 'i':
				iface = optarg; 
				if((chk_iface(iface) == -1))
					error_msg("Non existent interface given, "
				       "please check 'ifconfig -a' for available interfaces\n");
				else
					GOT_IFACE = 1;
				break;
			case 'n':
				num_pk = atoi(optarg);
				break;
			
			/* TCP/UDP Options */
			case 's':
				if((chk_config("SRC_PORT", optarg)) == -1)
					error_msg("Incorrect port number "
					        "provided, the correct format is as "
						"follows:\n\n -s 1-65535\n");
				break;
			case 'd':
				if((chk_config("DEST_PORT", optarg)) == -1)
					error_msg("Incorrect port number "
						"provided, the correct format is as "
						"follows:\n\n -d 1-65535\n");
				break;
			case 'f':
				if((chk_config("TCP_FLAGS", optarg)) == -1)		
					error_msg("Incorrect TCP flags selected"
					" please consult man pages for information\n");
				break;
			case 'w':
				if((chk_config("WINDOW", optarg)) == -1)
					error_msg("Incorrect window size specified," 
					"values between 0 and 65535 are accepted:\n\n");
				else
					printf("DEBUG -Window Size: %d\n",window);
				break;
			case 'u':
				if((chk_config("URG_POINTER", optarg)) == -1)
					error_msg("Incorrect prority pointer specified "
					"values between 0 and 3 are accepted\n");
				break;
			
			/* IP Options */
			case 'S':
				if((chk_config("SRC_IP", optarg)) == -1)
					error_msg("Incorrect ip address specified, " 
				                  "the correct format is as follows:\n\n"
			                          "-S 255.255.255.255\n");
				break;
			case 'D':
				if((chk_config("DEST_IP", optarg)) == -1)
					error_msg("Incorrect ip address specified, " 
				                  "the correct format is as follows:\n\n"
			                          "-D 255.255.255.255\n");
				break;
			case 'I':
				if((chk_config("IP_ID", optarg)) == -1)
					error_msg("Incorrect ip_id specified "
						"values between 0 and 65535 are accepted\n");
				break;
			case 'F':
				if((chk_config("FRAG_OPTS", optarg)) == -1)
					error_msg("Incorrect fragment option specified," 
			                	" available options are:\n\n"
				        	"r - Set reserved bit.\nd - Dont fragment."
				        	"\nm - More fragments.\n");	
				break;
			case 'L':
				if((chk_config("TTL", optarg)) == -1)
					error_msg("Invalid ttl (time to live),"
						"valid ttl is between 1-255\n"); 
				else
					printf("DEBUG -TTL: %d\n", ttl);
				break;
				
			/* ICMP Options */	
			case 't': /* imcp type  FIXME: */
				if((chk_config("ICMP_TYPE", optarg)) == -1)
					error_msg("Incorrect icmp type specified "
						"please consult man page for accepted types\n");
				break;
			case 'm': /* imcp mode  FIXME: */
				if((chk_config("ICMP_CODE", optarg)) == -1)
					error_msg("Incorrect icmp mode specified "
						"please consult man page for accepted modes\n");
				break;
			
			/* ARP/RARP Options */
			case 'o':
				if((chk_config("ARP_TYPE", optarg)) == -1)
					error_msg("Incorrect ARP TYPE specified, " 
				                  "please read the manual for details\n");
				break;
			case 'g':
				if((chk_config("SENDER_IP", optarg)) == -1)
					error_msg("Incorrect sender ip address specified, " 
				                  "the correct format is as follows:\n\n"
			                          "-g 255.255.255.255\n");
				break;
			case 'j':
				if((chk_config("TARGET_IP", optarg)) == -1)
					error_msg("Incorrect ip address specified, " 
				                  "the correct format is as follows:\n\n"
			                          "-h 255.255.255.255\n");
				break;
			case 'G':
				if((chk_config("SENDER_MAC", optarg)) == -1)
					error_msg("Incorrect mac address specified," 
				                  "the correct format is as follows:\n\n"
			                          "-G XX:XX:XX:XX:XX:XX\n");
				break;
			case 'J':
				if((chk_config("TARGET_MAC", optarg)) == -1)
					error_msg("Incorrect mac address specified," 
				                  "the correct format is as follows:\n\n"
			                          "-H XX:XX:XX:XX:XX:XX\n");
				break;
			
			/* Etherlink Options */
			case 'Z':
				if((chk_config("SRC_MAC", optarg)) == -1)
					error_msg("Incorrect mac address specified," 
				                  "the correct format is as follows:\n\n"
			                          "-Z XX:XX:XX:XX:XX:XX\n");	
				break;
			case 'Y':
				if((chk_config("DEST_MAC", optarg)) == -1)
					error_msg("Incorrect mac address specified," 
				                  "the correct format is as follows:\n\n"
		        	                  "-Y XX:XX:XX:XX:XX:XX\n");	
				break;
			case 'p':
				if((r_fd = open(optarg, O_RDONLY)) == -1)
					error_msg("Unable to open payload file.\n");
				payload = malloc(TCP_PAYLOAD_SZ);
				memset(payload, 0, sizeof(payload));
				if((pay_size = read(r_fd, payload, TCP_PAYLOAD_SZ+1)) == -1)
					error_msg("Unable to read file\n");
				if(pay_size > TCP_PAYLOAD_SZ)
					error_msg("Payload too large, maximum payload"
						  "size is 1380 bytes.\n");
				close(r_fd);
				break;
			default :
				 usage();
		}
	i++;
	}
	if(argc < 5)
		usage();
	
	if(HAVE_CONFIG == 1) get_config(config_file);
	if(WRITE_CONFIG == 1) write_config(write_file);
	
	if(IP_MODE == 1)
	{
		ether_hdr = build_ether_hdr(src_mac, dest_mac, ETHERTYPE_IP);
		
		ip_hdr = build_ip_hdr(src_addr, dest_addr, ttl, IPPROTO_IP, 
					frag_off, 0, pay_size, ip_id);
		
		packetlen = sizeof(struct ether_header) + ip_hdr->ip_len;
		packet = malloc(packetlen);
		memset(packet, 0, packetlen);
		
		memcpy(packet, ether_hdr, sizeof(struct ether_header));
		memcpy(packet + sizeof(struct ether_header), ip_hdr, sizeof(struct ip));	
		memcpy(packet + sizeof(struct ether_header) + sizeof(struct ip), 
				payload, pay_size);
		
		free(ether_hdr);
		free(ip_hdr);
	}
	
	if(TCP_MODE == 1)
	{
		/* build headers */
		ether_hdr = build_ether_hdr(src_mac, dest_mac, ETHERTYPE_IP);
		
		ip_hdr = build_ip_hdr(src_addr, dest_addr, ttl, IPPROTO_TCP, 
					frag_off, sizeof(struct tcphdr), pay_size, ip_id);
		tcp_hdr = build_tcp_hdr(src_port, dest_port, window, TCP_FIN,
					TCP_SYN, TCP_RST, TCP_PSH, TCP_ACK, TCP_URG,
					 u_priority, ip_hdr, payload, pay_size);
		
		/* Allocate memory */
		packetlen = sizeof(struct ether_header) + ip_hdr->ip_len;
		packet = malloc(packetlen);
		memset(packet, 0, packetlen);
		
		/* Construct packet in memory */
		memcpy(packet, ether_hdr, sizeof(struct ether_header));
		memcpy(packet + sizeof(struct ether_header), ip_hdr, sizeof(struct ip));
		memcpy(packet + sizeof(struct ether_header) + sizeof(struct ip), 
			tcp_hdr, sizeof(struct tcphdr));
		memcpy(packet + sizeof(struct ether_header) + sizeof(struct ip) + 
			sizeof(struct tcphdr), payload, pay_size);			
		
		free(ether_hdr);
		free(ip_hdr);
		free(tcp_hdr);
				
	}
	
	if(UDP_MODE == 1)
	{
		/* Build headers*/
		ether_hdr = build_ether_hdr(src_mac, dest_mac, ETHERTYPE_IP);
		
		ip_hdr = build_ip_hdr(src_addr, dest_addr, ttl, IPPROTO_UDP, 
					frag_off, sizeof(struct udphdr), pay_size, ip_id);
		udp_hdr = build_udp_hdr(src_port, dest_port, sizeof(struct udphdr), 
					pay_size, ip_hdr, payload);
				
		/* Allocate memory */
		packetlen = sizeof(struct ether_header) + ip_hdr->ip_len;
		packet = malloc(packetlen);
		memset(packet, 0, packetlen);
		
		/* Construct packet in memory */
		memcpy(packet, ether_hdr, sizeof(struct ether_header));
		memcpy(packet + sizeof(struct ether_header), ip_hdr, sizeof(struct ip));
		memcpy(packet + sizeof(struct ether_header) + sizeof(struct ip),
			udp_hdr, sizeof(struct udphdr));
		memcpy(packet + sizeof(struct ether_header) + sizeof(struct ip) +
			sizeof(struct udphdr), packet, pay_size);
		
		free(ether_hdr);
		free(ip_hdr);
		free(udp_hdr);
	}
	
	if(ICMP_MODE ==1)
	{	
		ether_hdr = build_ether_hdr(src_mac, dest_mac, ETHERTYPE_IP);
		
		ip_hdr = build_ip_hdr(src_addr, dest_addr, ttl, IPPROTO_ICMP, 
					frag_off, sizeof(struct icmphdr), pay_size, ip_id);
		icmp_hdr = build_icmp_hdr(icmp_type, icmp_code, icmp_id, icmp_seq, 
					 ip_hdr, payload, pay_size);
					
		packetlen = sizeof(struct ether_header) + ip_hdr->ip_len;
		packet = malloc(packetlen);
		memset(packet, 0, packetlen);
		
		memcpy(packet, ether_hdr, sizeof(struct ether_header));
		memcpy(packet + sizeof(struct ether_header), ip_hdr, sizeof(struct ip));
		memcpy(packet + sizeof(struct ether_header) + sizeof(struct ip),
			icmp_hdr, sizeof(struct icmphdr));
		memcpy(packet + sizeof(struct ether_header) + sizeof(struct ip) +
			sizeof(struct icmphdr), packet, pay_size);
		
		free(ether_hdr);
		free(ip_hdr);
		free(icmp_hdr);
	}
	
	if(ARP_MODE == 1)
	{  	
		ether_hdr = build_ether_hdr(src_mac, dest_mac, ETHERTYPE_ARP);
		
		arp_hdr = build_arp_hdr(arp_optc);
		
		arp_ip_hdr = build_arp_ethip(send_mac, send_ip, targ_mac, targ_ip);
		
		packetlen = sizeof(struct ether_header) + sizeof(struct arphdr) + pay_size;
		packetlen = packetlen + sizeof(struct arp_ethip_hdr);
			
		packet = malloc(packetlen);
		memset(packet, 0, packetlen);
		
		memcpy(packet, ether_hdr, sizeof(struct ether_header));
		memcpy(packet + sizeof(struct ether_header), arp_hdr, sizeof(struct arphdr));
		memcpy(packet + sizeof(struct ether_header) + sizeof(struct arphdr), arp_ip_hdr,
			sizeof(struct arp_ethip_hdr));
		memcpy(packet + sizeof(struct ether_header) + sizeof(struct arphdr) +
			sizeof(struct arp_ethip_hdr), packet, pay_size);
		
		free(ether_hdr);
		free(arp_hdr);
		free(arp_ip_hdr);
	}
	
	if(GOT_IFACE == 0)
		rsock = raw_connect(rsock, dest_addr, dest_port, ip_hdr->ip_p);
		//rsock = bind_raw_local(rsock, dest_addr, ip_hdr->ip_p);
	else
		rsock = sock_to_iface(iface, rsock, ETH_P_ALL);
	
	while(num_pk > 0)
	{
		if((write(rsock, (void *)packet, packetlen)) == -1)
			perror("Write");
		num_pk--;
	}
	close(rsock);
	free(packet);


	return 0;
}
