/*
 * raw_stuff.h  -dlevette
 *
 * Function prototypes
*/
#include <netinet/ip.h>
#include <netinet/ether.h>

#define APP      "pkmap"
#define VERSION  "0.1"
#define ETHER_LEN strlen("00:00:00:00:00:00")

#define TCP_PAYLOAD_SZ 1380
#define MTU 1500

#define MAXLINE 256

/* Configuration stuff */
struct config_entrys
{
	char *option;
	char *value;
};

struct config_entrys *get_default_config();
void get_config(char *file);
int chk_config(char *option, char *value);
int create_config(char *option, char *value);
int write_config(char *write_file);

/* Validation routines */
int chk_mac(char *mac, int len);
int chk_ip(char *ip);
int chk_iface(char *iface);
int chk_int(int min, int max, int value);
int assign_tcp_flags(char *flags);
int assign_frag_opts(char *opts);
char *chomp(char *string);
char *unchomp(char *string);

/* Header Construction routines */
struct ether_header *build_ether_hdr(char *src_mac, char *dest_mac, int ether_proto);

struct ip *build_ip_hdr(char *src_addr, char *dest_addr, u_short ttl, 
			  char proto, int frag_off, int trans_sz, int payload_sz, u_short ip_id);

struct tcphdr *build_tcp_hdr(u_short src_port, u_short dest_port, 
			     u_short win_sz, int TCP_FIN, int TCP_SYN, int TCP_RST,
			     int TCP_PSH, int TCP_ACK, int TCP_URG, u_short urg, 
			    struct ip *ip_hdr, char *payload, int payload_sz);

struct udphdr *build_udp_hdr(u_short src_port, u_short dest_port, 
			     	int trans_sz, int payload_sz, struct ip *ip_hdr, 
					char *payload);

struct icmphdr *build_icmp_hdr(u_short icmp_type, u_short icmp_code,
				u_short icmp_ip, u_short icmp_seq, struct ip *ip_hdr, 
					char *payload, int payload_sz);

struct arphdr *build_arp_hdr(u_short arp_optc);

struct arp_ethip_hdr *build_arp_ethip(char *send_mac, char *send_ip, char *targ_mac,
					char *targ_ip);

struct pseudo *build_pseudo(struct ip *ip_hdr);

struct pseudo
{
	struct  in_addr src_ip, dest_ip;
	u_char  reserved;
	u_char  protocol;
	u_short len;
};

struct arp_ethip_hdr
{
	u_char sender_mac[ETH_ALEN];
	u_char target_mac[ETH_ALEN];
	struct  in_addr sender_ip, target_ip;
};

	
/* Checksum routine */
u_short calc_csum(u_char *data, int len);

/* Socket routines */
int open_raw_sock(int proto);
int sock_to_iface(char *iface, int rsock, int proto);
int raw_connect(int rsock, char *REMOTE_ADDR, int PORT, int proto);
int bind_raw_local(int rsock, int PORT, int proto);

/* Help and error routines */
void help();
void ip_help();
void tcp_help();
void udp_help();
void icmp_help();
void arp_help();
void error_msg(char *msg);
