/*
 * validate.c  -dlevette
 *
 * Input validation routines
*/
#include <stdio.h>
#include <stdlib.h>

#include <ctype.h>
#include <string.h>
#include <unistd.h>

#include <net/if.h>
#include <sys/ioctl.h>
#include <linux/if_packet.h>
#include <net/ethernet.h>
#include <sys/socket.h>
#include "raw_stuff.h"

/* 
 * Check mac address format 
 *
*/
int chk_mac(char *mac, int len)
{
	int i = 0, flag = 0;
	if(strlen(mac) != len)
		return -1;
	while(i < len){
		tolower(mac[i]);
		if((isxdigit(mac[i]) != 0))
			flag++;
		else if((mac[i] == ':') && (flag == 2))
			flag = 0;
		else
			return -1;
	i++;
	}
	return 0;
}

/*
 * Check ip address correct format and in range
 *
*/
int chk_ip(char *ip)
{
	int a, b, c, d = 256;
	sscanf(ip, "%d %*c %d %*c %d %*c %d", &a, &b, &c, &d);
	if(a < 0 || a > 255) return -1;
	if(b < 0 || b > 255) return -1;
	if(c < 0 || c > 255) return -1;
	if(d < 0 || d > 255) return -1;
	else return 0;
		
}

/*
 * Check network interface exists
 *
*/
int chk_iface(char *iface)
{
	struct ifreq ifr;
	int rsock;

	rsock = open_raw_sock(ETH_P_ALL);

	strncpy(ifr.ifr_name, iface, IF_NAMESIZE);
	if(ioctl(rsock,SIOCGIFFLAGS, &ifr) < 0)
		return -1;
	else{
		close(rsock);
		return 0;
	}
}

/*
 * Verify integer in range
*/
int chk_int(int min, int max, int value)
{
	if((value < min || value > max))
		return -1;
	else
		return 0;
}

/*
 * Replace newline with '\0'
 *
*/
char *chomp(char *string)
{
	int len = strlen(string);
	if( string[len-1] == '\n')
		string[len-1] = '\0';
	
	return(string);
}

/*
 * Replace '\0' with newline
 *
*/
char *unchomp(char *string)
{
	int len = strlen(string);
	if(string[len-1] == '\0')
		string[len-1] = '\n';
	
	return(string);
}
